"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createResolver = void 0;
var tslib_1 = require("tslib");
var enhanced_resolve_1 = require("enhanced-resolve");
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = require("path");
var typescript_1 = require("typescript");
var logger_1 = require("./logger");
/**
 * Create the dependencies resolver;
 * @internal
 * @param cwd - working dir
 * @param compilerOptions - compiler options
 * @param options - resolver options
 * @param logger - logger
 */
function createResolver(cwd, compilerOptions, options, logger) {
    /**
     * Prepare compiler host.
     */
    var compilerHost = (0, typescript_1.createCompilerHost)(compilerOptions);
    logger.debug('create resolver with options:', options);
    /**
     * Prepare base resolver
     */
    var resolver = enhanced_resolve_1.ResolverFactory.createResolver(tslib_1.__assign({ cacheWithContext: false, fileSystem: new enhanced_resolve_1.CachedInputFileSystem(fs_1.default, 4000), useSyncFileSystemCalls: true }, options));
    var externalMatch = function (target) { return function (val) {
        if (typeof val === 'string') {
            return val === target;
        }
        else {
            return val.test(target);
        }
    }; };
    /**
     * Is checking alias is a absolute url.
     * @param target - target dependency
     */
    var checkAliasForUrl = function (target) {
        var _a;
        var resolution;
        if (Array.isArray(options.alias)) {
            var alias = options.alias.find(function (i) { return i.name === target; });
            if (alias && typeof alias.alias === 'string') {
                resolution = alias.alias;
            }
        }
        else {
            var alias = (_a = options.alias) === null || _a === void 0 ? void 0 : _a[target];
            if (alias && typeof alias === 'string') {
                resolution = alias;
            }
        }
        if (resolution && (resolution.startsWith('http://') || resolution.startsWith('https://'))) {
            return resolution;
        }
    };
    return function (context, target) {
        var _a;
        var prefix = "import of: '".concat(target, "'\n from: '").concat(context, "\n");
        // is external
        if (options.externals && options.externals.find(externalMatch(target))) {
            logger.debug("".concat(prefix, " resolved as external"));
            return { context: context, target: target, resolution: target, isExternal: true };
        }
        // is a absolute url
        var urlAlias = checkAliasForUrl(target);
        if (urlAlias) {
            logger.debug("".concat(prefix, " resolved by alias to: ").concat(urlAlias));
            return { context: context, target: target, resolution: urlAlias, isUrl: true };
        }
        // is from ts/tsx module
        if (context.endsWith('.ts') || context.endsWith('.tsx')) {
            // try resolving by the typescript
            var tsResolution = (0, typescript_1.nodeModuleNameResolver)(target, context, compilerOptions, compilerHost);
            if ((_a = tsResolution === null || tsResolution === void 0 ? void 0 : tsResolution.resolvedModule) === null || _a === void 0 ? void 0 : _a.resolvedFileName) {
                if (tsResolution.resolvedModule.extension !== typescript_1.Extension.Dts) {
                    logger.debug("".concat(prefix, " resolved by typescript to: ").concat(tsResolution.resolvedModule.resolvedFileName));
                    return {
                        isTypescript: true,
                        context: context,
                        target: target,
                        resolution: tsResolution.resolvedModule.resolvedFileName
                    };
                }
            }
        }
        try {
            // try resolving by the node
            var nodeResolution = resolver.resolveSync({}, (0, path_1.dirname)(context), target);
            if (nodeResolution) {
                logger.debug("".concat(prefix, " resolved by enhanced-resolver to: ").concat(nodeResolution));
                return {
                    isNode: true,
                    context: context,
                    target: target,
                    resolution: nodeResolution
                };
            }
            // eslint-disable-next-line no-empty
        }
        catch (e) {
            logger.debug(logger_1.loggerPrefix, 'enhanced-resolver error:', e);
        }
        logger.error("".concat(prefix, " not resolved !!!"));
        throw new Error("".concat(logger_1.loggerPrefix, " resolution not found for ").concat(target, " imported by ").concat(context));
    };
}
exports.createResolver = createResolver;
//# sourceMappingURL=create-resolver.js.map